@@ -173,10 +173,11 @@ also accepted as dictionary entries here
             self.STRING_WITH_NICKNAME = self.STRING
 
     def __str__(self):
-        string="/* Autogenerated by waf */\n"
-        string+="#define SAMBA_VERSION_MAJOR %u\n" % self.MAJOR
-        string+="#define SAMBA_VERSION_MINOR %u\n" % self.MINOR
-        string+="#define SAMBA_VERSION_RELEASE %u\n" % self.RELEASE
+        string="/* Autogenerated by waf */\n" +\
+                "#define SAMBA_COPYRIGHT_STRING \"%s\"\n" % self.SAMBA_COPYRIGHT_STRING +\
+                "#define SAMBA_VERSION_MAJOR %u\n" % self.MAJOR +\
+                "#define SAMBA_VERSION_MINOR %u\n" % self.MINOR +\
+                "#define SAMBA_VERSION_RELEASE %u\n" % self.RELEASE
         if self.REVISION is not None:
             string+="#define SAMBA_VERSION_REVISION %u\n" % self.REVISION
 
@@ -235,22 +236,27 @@ also accepted as dictionary entries here
 def samba_version_file(version_file, path, env=None, is_install=True):
     '''Parse the version information from a VERSION file'''
 
-    f = open(version_file, 'r')
-    version_dict = {}
-    for line in f:
-        line = line.strip()
-        if line == '':
-            continue
-        if line.startswith("#"):
-            continue
-        try:
-            split_line = line.split("=")
-            if split_line[1] != "":
-                value = split_line[1].strip('"')
-                version_dict[split_line[0]] = value
-        except:
-            print("Failed to parse line %s from %s" % (line, version_file))
-            raise
+    with open(version_file, 'r') as f:
+        version_dict = {}
+        for line in f:
+            line = line.strip()
+            if line == '':
+                continue
+            if line.startswith("#"):
+                continue
+            try:
+                split_line = line.split("=")
+                if split_line[1] != "":
+                    value = split_line[1].strip('"')
+                    version_dict[split_line[0]] = value
+            except:
+                print("Failed to parse line %s from %s" % (line, version_file))
+                raise
+
+    if "SAMBA_VERSION_VENDOR_SUFFIX" in env:
+        version_dict["SAMBA_VERSION_VENDOR_SUFFIX"] = env.SAMBA_VERSION_VENDOR_SUFFIX
+    if "SAMBA_VERSION_VENDOR_PATCH" in env:
+        version_dict["SAMBA_VERSION_VENDOR_PATCH"] = str(env.SAMBA_VERSION_VENDOR_PATCH)
 
     return SambaVersion(version_dict, path, env=env, is_install=is_install)
 
